/*
 BreakCmd.cc - originally InterruptibleCmd
 modified by Will 18, 2000: will@opaline.demon.co.uk

 Adapted from 
 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall.
*/

#include "BreakCmd.h"
#include "WorkingDialogManager.h"
#include "Application.h"
#include <Xm/Xm.h>
#include <Xm/MessageB.h>
#include <assert.h>


BreakCmd::BreakCmd ( const char *name, int active ) : NoUndoCmd ( name, active )
{
	_wpId = ( unsigned long ) NULL;	// no typecast in Young
	_callback = NULL;	// callbacks are specified in execute()
	_clientData = NULL;
	_done = FALSE;
}


BreakCmd::~BreakCmd()
{
	if ( _wpId )
		XtRemoveWorkProc ( _wpId );
}


/*
  BreakCmd defines two execute() functions; the first allows specification of a
  function to be called on task completion, the second overrides that inherited
  from Cmd. The callback function passed to execute() mut have the form
  void callback ( BreakCmd *, Boolean interrupted, void * )
  After savinga pointer to the callback function, the first execute() calls the
  second, which takes no arguments.
*/

void BreakCmd::execute ( TaskDoneCallback callback, void *clientData )
{
	_callback = callback;
	_clientData = clientData;
	execute();
}

/*
  The second execute(), which cab called directly if there is no callback to be
  registered, sets _done FALSE and calls Cmd::execute(), which eventually calls
  doCommand() - which latter must be provided by classes derived from BreakCmd
  
  execute() checks _done before posting a message and installing a work
  procedure. If the task is done, cleanup() notifies derived classes, then the
  _callback function is called to notify the application initiating this command.
  
  If the task is incomplete, a working dialog is posted and a work proc
  installed.
  
  post() arguments specify a pointer to the current BreakCmd instance and a
  pointer to the interruptCallback() member function, which will be registered as
  the BusyDialogManager's cancel callback.
*/

void BreakCmd::execute()
{
	_done = FALSE;
	Cmd::execute();	// handle Undo and other generalities of Cmd; calls doCommand()
	
	if ( _done )
	{	
		cleanup();
		
		if ( _callback )
			( *_callback ) ( this, FALSE, _clientData );
	}
	else
	{
		workingDialogManager->post ( "", ( void * ) this, NULL,
									  &BreakCmd::interruptCallback );

		_wpId = XtAppAddWorkProc ( application->appContext(),
								   &BreakCmd::workProcCallback, ( XtPointer ) this );

	}
}


/*
  workProcCallback() is a static member function registered with XT as a work
  procedure. This fn retrieves the instance pointer from the cklient data and
  calls workProc(). A work procedure must return FALSE if task incomplete, else
  TRUE if it should be removed.
*/

Boolean BreakCmd::workProcCallback ( XtPointer clientData )
{
	BreakCmd *obj = ( BreakCmd * ) clientData;
	return ( obj->workProc() );	// return value returned by workProc()
}

/*
  workproc() calls doCommand(). If a task is complete following this call,
  workProc() removes the scren dialog and calls a virutal cleanup(), finally
  calling the application-defined callback function. The first arg identifies the
  current BreakCmd object, the second the task status. Returning _done as TRUE
  causes XT to remove the workProcCallback() wrok procedure, otherwise the
  callback is called again after Xt has cleared the event queue of pending
  events.
*/

Boolean BreakCmd::workProc()
{
	doCommand();
	
	if ( _done )
	{
		workingDialogManager->unpost();
		cleanup();
		
		if ( _callback )
			( *_callback ) ( this, FALSE, _clientData );
	}
	
	return ( _done );
}


void BreakCmd::cleanup()
{
	// empty - may be overriden by derived classes
}


void BreakCmd::interruptCallback ( void *clientData )
{
	BreakCmd *obj = ( BreakCmd * ) clientData;
	obj->interrupt();
}


void BreakCmd::interrupt()
{
	XtRemoveWorkProc ( _wpId );
	workingDialogManager->unpost();
	cleanup();
	
	if ( _callback )
		( *_callback ) ( this, TRUE, _clientData );
}


void BreakCmd::updateMessage( char *msg )
{
	workingDialogManager->updateMessage ( msg );
}

void BreakCmd::updateTimeout ( unsigned long progressInterval )
{
	workingDialogManager->updateTimeout ( progressInterval );
}
